/**********************************************************************\
**
**                         Copyright (c) 2012-2018 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Xpm_Api.h
**  Facility:    Xpm application programming interface (API) for linux
**  Version:     R4-V1.03b-BETA - 2018AUG16
**  Environment: Linux Debian
**  Compiler:    gcc, use ./build to compile
**  Abstract:    Api Library header file
**  History:     R1-V1_00  / 2010JAN25 / TKA / Initial release
**               R2-V1.01a / 2012FEB14 / TKA / New API communication added (libusb)
**               R2-V1.01a / 2012FEB14 / TKA / New Error codes of application interface
**               R2-V1.01a / 2012FEB14 / TKA / Printer Ticket request packet added for ticket monitoring
**               R2-V1.01a / 2012FEB14 / TKA / Configuration Data - Paper Path Options extended
**               R2-V1.01a / 2012FEB14 / TKA / Configuration data - customer sensors extended
**               R2-V1.01a / 2012FEB14 / TKA / Configuration data structure extended
**               R3-V1.02  / 2017MAY24 / TKA / Recompiled for x86/x64  and arm.
**                                  - Added new macros and adapted to x64 bit.
**               R4-V1.03a / 2018AUG01 / TKA / FIFO buffer implemented
**                                             USB2.0 preparation 
**               R4-V1.03b / 2018AUG16 / TKA / Return value of printer handle change from 'int' to 'intptr_t' 
**                                             due to 32/64bit incompatibilities on some 64bit distributions (Ubuntu 18.04-64bit) 
**
\**********************************************************************/

#ifndef __XPM_API_H
#define __XPM_API_H

/*
** INCLUDES
*/

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
  
/*
** MACROS
*/

//-- Error codes of application programming interface
#define D_XPM_API_OPEN_FAILED                                              -1
#define D_XPM_API_INVALID_DEVICE                                           -2
#define D_XPM_API_DETACH_FAILED                                            -3
#define D_XPM_API_CLAIM_FAILED                                             -4
#define D_XPM_API_GET_ACTIVE_CONFIG_FAILED                                 -5
#define D_XPM_API_NO_INTERFACE                                             -6
#define D_XPM_API_ENDPOINT_COUNT_FAILURE                                   -7
#define D_XPM_API_COMM_BUFFER_FAILURE                                      -8

//-- Error codes of "exo_api_printer_read" interface
#define D_XPM_API_READ_BUFFER_FAILURE                                      -1
#define D_XPM_API_READ_SAVING_FAILURE                                      -2
#define D_XPM_API_READ_OVERFLOW                                            -3

//-- Data packets id's which correspond with parameter ubId in TS_XPM_API_PCKT_HEADER
//.. Note that the packet header "ESC [FF]+[i]+[n]" has [i] as packet identifier and [n] as packet data size.
//.. The packet data "[d1]+[dn]" immediately follows the packet header.
#define D_XPM_API_PID_PRINTER_STATUS                                          2
#define D_XPM_API_PID_PRINTER_INFO                                            3
#define D_XPM_API_PID_PRINTER_SENSORS                                         4
#define D_XPM_API_PID_PRINTER_FLASH                                           5
#define D_XPM_API_PID_PRINTER_FEED                                            6
#define D_XPM_API_PID_PRINTER_BARCODE                                         7
#define D_XPM_API_PID_PRINTER_PRESENT                                         8
#define D_XPM_API_PID_PRINTER_LIFE                                            9
#define D_XPM_API_PID_PRINTER_TICKET                                          10  // Ticket monitoring information

//-- Data request id's which correspond with bits in parameter [n] in emulation command "GS A+[n1]+[n2]"
//.. After reception of this command, the printer will send a sequence of requested data packets
//.. The packet data "[d1]+[dn]" immediately follows the packet header.
#define D_XPM_API_RID_PRINTER_STATUS                                          0x0001
#define D_XPM_API_RID_PRINTER_INFO                                            0x0002
#define D_XPM_API_RID_PRINTER_SENSORS                                         0x0004
#define D_XPM_API_RID_PRINTER_FLASH                                           0x0008
#define D_XPM_API_RID_PRINTER_FEED                                            0x0010
#define D_XPM_API_RID_PRINTER_BARCODE                                         0x0020
#define D_XPM_API_RID_PRINTER_PRESENT                                         0x0040
#define D_XPM_API_RID_PRINTER_LIFE                                            0x0080
#define D_XPM_API_RID_PRINTER_BARCODES                                        0x0100 // Multi barcodes
#define D_XPM_API_RID_PRINTER_TICKET                                          0x0200 // Ticket monitoring information

//-- Printer status packet -> printer status dword bits
#define D_XPM_API_PSDW_PRINTER_ERROR                                          0x00000001
#define D_XPM_API_PSDW_PRINT_HEAD_TEMP_ALERT                                  0x00000002
#define D_XPM_API_PSDW_FLASH_CHECKSUM_ERROR                                   0x00000004
#define D_XPM_API_PSDW_PRINTER_STALLED                                        0x00000008
#define D_XPM_API_PSDW_PAPER_OUT_DETECTED                                     0x00000010
#define D_XPM_API_PSDW_PAPER_LOW_A_DETECTED                                   0x00000020
#define D_XPM_API_PSDW_PAPER_MARK_DETECTED                                    0x00000040
#define D_XPM_API_PSDW_PAPER_IN_CHUTE_DETECTED                                0x00000080
#define D_XPM_API_PSDW_PRINT_HEAD_UP_DETECTED                                 0x00000100
#define D_XPM_API_PSDW_LAST_PRINTOUT_LOST                                     0x00000200
#define D_XPM_API_PSDW_AUXILIARY_SENSOR_ACTIVE                                0x00000400
#define D_XPM_API_PSDW_SYSTEM_FAILURE                                         0x00000800
#define D_XPM_API_PSDW_POWER_FAILURE                                          0x00001000
#define D_XPM_API_PSDW_CUSTOMER_FLAG_1                                        0x00002000
#define D_XPM_API_PSDW_CUSTOMER_FLAG_2                                        0x00004000
#define D_XPM_API_PSDW_PRINTER_NOT_READY                                      0x00008000
#define D_XPM_API_PSDW_PAPER_LOW_B_DETECTED                                   0x00010000
#define D_XPM_API_PSDW_BARCODE_SCANNED                                        0x00020000
#define D_XPM_API_PSDW_PRINT_UNIT_EMTPY                                       0x00040000

//-- Printer status packet -> Printer error word codes, low byte is WHAT, high byte is WHEN
#define D_XPM_API_PECW_WHAT_PRINTER_JAMMED                                    1
#define D_XPM_API_PECW_WHAT_CUTTER_JAMMED                                     2
#define D_XPM_API_PECW_WHAT_FEEDER_JAMMED                                     3
#define D_XPM_API_PECW_WHAT_PRESENTER_JAMMED                                  4
#define D_XPM_API_PECW_WHEN_WHILE_PRINTING                                    1
#define D_XPM_API_PECW_WHEN_WHILE_LOADING_PAPER                               2
#define D_XPM_API_PECW_WHEN_WHILE_ENDING_PAPER                                3
#define D_XPM_API_PECW_WHEN_WHILE_INITIALIZING                                4
#define D_XPM_API_PECW_WHEN_WHILE_ENDING_PRINT                                5
#define D_XPM_API_PECW_WHEN_WHILE_IDLE                                        6
#define D_XPM_API_PECW_WHEN_WHILE_RETRACTING_PAPER                            7
#define D_XPM_API_PECW_WHEN_WHILE_ACTIVATING_PAPER                            8
#define D_XPM_API_PECW_WHEN_WHILE_PARKING_PAPER                               9
#define D_XPM_API_PECW_WHEN_WHILE_EMPTY                                       10
#define D_XPM_API_PECW_WHEN_WHILE_EJECTING_PRINT                              11
#define D_XPM_API_PECW_WHEN_WHILE_INVALIDATING_PAPER                          12
#define D_XPM_API_PECW_WHEN_WHILE_LOADING_PRINT                               13
#define D_XPM_API_PECW_WHEN_WHILE_PRESENTING_PRINT                            14
#define D_XPM_API_PECW_WHEN_WHILE_REJECTING_PRINT                             15
#define D_XPM_API_PECW_WHEN_WHILE_RETRACTING_PRINT                            16
#define D_XPM_API_PECW_WHEN_WHILE_CANCELLING_PRINT                            17
#define D_XPM_API_PECW_WHEN_WHILE_UNJAMMING                                   18
#define D_XPM_API_PECW_WHAT_ERROR(ERR,WHAT)                                   (((ERR)&0xFF00) == (((WHAT) & 0x00FF) << 8))
#define D_XPM_API_PECW_WHEN_ERROR(ERR,WHEN)                                   (((ERR)&0x00FF) == ((WHEN) & 0x00FF))

//-- Printer status packet -> Printer error word information, enumeration  
#define D_XPM_API_PEIW_NONE                                                                                                   0 
//.. Print unit 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_INVALID_PAPER_CODE_IN_EEPROM_STORAGE                                        1000 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_OUSIDE_PARK_AREA_WHEREAS_UNJAMMING_ON_INIT_IS_DISABLED                1001 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_COULD_NOT_RETRACT_PAPER_INTO_PRINT_UNIT                                     1002 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_DETECTING_TOP_OF_PAPER           1003 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINS_OCCUPIED_WHILE_DETECTING_TOP_OF_PAPER             1004 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_ENTRY_SENSOR_WAS_UNOCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER              1005 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINED_OCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER            1006 
#define D_XPM_API_PEIW_PRINTER_x_INITIALIZATION_x_PAPER_EXIT_SENSOR_REMAINED_UNOCCUPIED_WHILE_INQUIRING_TOP_OF_PAPER          1007 
#define D_XPM_API_PEIW_PRINTER_x_IDLE_x_CUTTER_JAMMED_WHILE_BEING_IDLE                                                        1008 
#define D_XPM_API_PEIW_PRINTER_x_PRINTING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_PRINT                                       1009 
#define D_XPM_API_PEIW_PRINTER_x_ACTIVATING_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED_WHILE_ACTIVATING_PAPER                          1010 
#define D_XPM_API_PEIW_PRINTER_x_ACTIVATING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_ACTIVATING_PAPER                           1011 
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_WAS_OCCUPIED_WHILE_LOADING_PAPER                                       1012 
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_ENTRY_SENSOR_WAS_UNOCCUPIED_WHILE_LOADING_PAPER                                    1013 
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_WAS_OCCUPIED_TOO_EARLY_WHILE_LOADING_PAPER                             1014 
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_WHILE_LOADING_PAPER                                 1015 
#define D_XPM_API_PEIW_PRINTER_x_LOADING_x_CUTTER_JAMMED_DURING_TEST                                                          1016 
#define D_XPM_API_PEIW_PRINTER_x_ENDING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                         1017 
#define D_XPM_API_PEIW_PRINTER_x_CANCELLING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                     1018 
#define D_XPM_API_PEIW_PRINTER_x_CANCELLING_x_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_EJECT                                     1019 
#define D_XPM_API_PEIW_PRINTER_x_RETRACTING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                     1020 
#define D_XPM_API_PEIW_PRINTER_x_EJECTING_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                                       1021 
//.. Cutter unit 
#define D_XPM_API_PEIW_CUTTER_x_INITIALIZATION_x_CUTTER_JAMMED_DURING_INITIALIZATION                                          2000 
#define D_XPM_API_PEIW_CUTTER_x_INITIALIZATION_x_CUTTER_JAMMED_DURING_TEST                                                    2001 
#define D_XPM_API_PEIW_CUTTER_x_LOADING_x_CUTTER_JAMMED_DURING_PAPER_LOAD                                                     2002 
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_AT_END_OF_FORM                                                        2003 
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_AT_END_OF_PRINT                                                       2004 
#define D_XPM_API_PEIW_CUTTER_x_PARKED_x_CUTTER_JAMMED_WHILE_BEING_IDLE                                                       2005  
#define D_XPM_API_PEIW_CUTTER_x_UNJAMMING_x_CUTTER_JAMMED_DURING_UNJAMMING                                                    2006 
#define D_XPM_API_PEIW_CUTTER_x_CUTTING_x_CUTTER_JAMMED_DURING_TEST                                                           2007 
//.. Feed unit, dual 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_LOADING_x_PRINT_UNIT_REFUSED_TO_LOAD_PAPER                                               3000 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_LOADING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                          3001 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                       3002 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_ACTIVATION                  3003 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_PRINT_UNIT_ENTRY_SENSOR_REMAINS_UNOCCUPIED_DURING_ACTIVATION                3004 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_ACTIVATING_x_PRINT_UNIT_REFUSED_TO_ACTIVATE_PAPER                                        3005 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_UNOCCUPIED_DURING_PARKING                        3006 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_PRINTER_UNIT_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_PARKING                      3007 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_PARKING                          3008 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_PRINT_UNIT_REFUSED_TO_RETRACT_PAPER                                            3009 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_PARKING_x_FEED_UNIT_FAILED_TO_RETRACT_PAPER_FROM_THE_PRINT_UNIT                          3010 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_PRINTER_UNIT_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                   3011 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_FEED_UNIT_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_RETRACT                       3012 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_PRINT_UNIT_REFUSED_TO_RETRACT_PAPER                                         3013 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_RETRACTING_x_FEED_UNIT_FAILED_TO_RETRACT_PAPER_FROM_THE_PRINT_UNIT                       3014 
#define D_XPM_API_PEIW_FEEDER_DUAL_x_INITIALIZATION_x_PRINT_UNIT_HAS_MULTIPLE_PAPER_PATHS_COUPLED                             3015 
//.. Presenter unit => METIS 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                     4000 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_BUNDLER_SENSOR_DID_NOT_DETECT_PAPER                                 4001 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_REJECT_SENSOR_DID_NOT_DETECT_PAPER                                  4002 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_REJECT_SENSOR_REMAINS_OCCUPIED                                      4003 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_INITIALIZATION_x_BUNDLER_SENSOR_REMAINS_OCCUPIED                                     4004 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_BUNDLER_CAPACITY_EXHAUSTED                                                 4005 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_DECEEDS_MINIMUM                                               4006 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                          4007 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_EXCEEDS_MAXIMUM                                               4008 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_LOADING_x_PAPER_LENGTH_MISMATCHES_BUNDLED_PAGES                                      4009 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_PARKING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                   4010 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                  4011 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_UNOCCUPIED                                  4012 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_EJECTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                    4013 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_RETRACTING_BUNDLE_x_PRESENT_SENSOR_REMAINS_OCCUPIED                                  4014 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_BUNDLER_SENSOR_REMAINS_UNOCCUPIED                                 4015 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_REJECT_SENSOR_REMAINS_UNOCCUPIED                                  4016 
#define D_XPM_API_PEIW_PRESENTER_METIS_x_REJECTING_BUNDLE_x_REJECT_SENSOR_REMAINS_OCCUPIED                                    4017 
//.. Presenter unit => BA  
#define D_XPM_API_PEIW_PRESENTER_BA_x_LOADING_PRINT_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED                                         4100 
#define D_XPM_API_PEIW_PRESENTER_BA_x_LOADING_PRINT_x_EXIT_SENSOR_REMAINS_UNOCCUPIED                                          4101 
#define D_XPM_API_PEIW_PRESENTER_BA_x_EJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED                                          4102 
#define D_XPM_API_PEIW_PRESENTER_BA_x_EJECTING_PRINT_x_EXIT_SENSOR_REMAINS_OCCUPIED                                           4103 
//.. Presenter unit => LOOP/RETRACT 
#define D_XPM_API_PEIW_PRESENTER_LR_x_UNDEFINED_x_FLAP_SENSOR_REMAINS_OCCUPIED_AFTER_OPENING                                  4200 
#define D_XPM_API_PEIW_PRESENTER_LR_x_UNDEFINED_x_FLAP_SENSOR_REMAINS_FREE_AFTER_CLOSING                                      4201 
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_EXIT_SENSOR_REMAINS_OCCUPIED_AFTER_RETRACT                             4202 
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_EXIT_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                             4203 
#define D_XPM_API_PEIW_PRESENTER_LR_x_INITIALIZATION_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                            4204 
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_ENTRY_SENSOR_REMAINS_UNOCCUPIED                                         4205 
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_EXIT_SENSOR_REMAINS_UNOCCUPIED                                          4206 
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_PAGE_LENGTH_DECEEDS_MINIMUM                                             4207 
#define D_XPM_API_PEIW_PRESENTER_LR_x_EJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_EJECT                             4208 
#define D_XPM_API_PEIW_PRESENTER_LR_x_RETRACTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_EJECT                           4209 
#define D_XPM_API_PEIW_PRESENTER_LR_x_REJECTING_PRINT_x_ENTRY_SENSOR_REMAINS_OCCUPIED_DURING_REJECT                           4210 
#define D_XPM_API_PEIW_PRESENTER_LR_x_LOADING_PRINT_x_ENTRY_SENSOR_OCCUPIED_TOO_EARLY                                         4211 


//-- Printer sensor packet -> printer sensor state 
#define D_XPM_API_PSS_MSN_NONE                                                0x00
#define D_XPM_API_PSS_MSN_REFLEX                                              0x10
#define D_XPM_API_PSS_MSN_TLIGHT                                              0x20
#define D_XPM_API_PSS_MSN_SWITCH                                              0x30
#define D_XPM_API_PSS_LSN_NONE                                                0x00
#define D_XPM_API_PSS_LSN_NO_PAPER                                            0x01
#define D_XPM_API_PSS_LSN_PAPER                                               0x02
#define D_XPM_API_PSS_LSN_PARKED                                              0x03
#define D_XPM_API_PSS_LSN_CUTTING                                             0x04
#define D_XPM_API_PSS_LSN_HEAD_DOWN                                           0x05
#define D_XPM_API_PSS_LSN_HEAD_UP                                             0x06
#define D_XPM_API_PSS_LSN_ACTIVE                                              0x07
#define D_XPM_API_PSS_LSN_INACTIVE                                            0x08
#define D_XPM_API_PSS_LSN_PRESSED                                             0x09
#define D_XPM_API_PSS_LSN_RELEASED                                            0x0A

//-- Printer flash packet -> printer flash area index/state (high/low nibble)
#define D_XPM_API_PFAI_FIRMWARE                                               0x00
#define D_XPM_API_PFAI_FONT_1                                                 0x01
#define D_XPM_API_PFAI_LOGO                                                   0x02
#define D_XPM_API_PFAI_FONT_2                                                 0x03
#define D_XPM_API_PFAI_WIDE_FONT                                              0x04
#define D_XPM_API_PFAI_IMAGES                                                 0x05
#define D_XPM_API_PFAS_UNDEFINED                                              0x00
#define D_XPM_API_PFAS_OK                                                     0x01
#define D_XPM_API_PFAS_ERROR_CRC                                              0x02
#define D_XPM_API_PFAS_ERROR_TYPE                                             0x03
#define D_XPM_API_PFAS_ERROR_SIZE                                             0x04

//-- Printer feed packet -> printer feed status dword bits
#define D_XPM_API_PFSW_ERROR                                                  0x0001
#define D_XPM_API_PFSW_ACTIVE                                                 0x0002
#define D_XPM_API_PFSW_PARKED                                                 0x0004
#define D_XPM_API_PFSW_PAPER_OUT_DETECTED                                     0x0008
#define D_XPM_API_PFSW_PAPER_LOW_A_DETECTED                                   0x0010
#define D_XPM_API_PFSW_PAPER_LOW_B_DETECTED                                   0x0020

//-- Presenter status packet -> printer present status word
#define D_XPM_API_PPSW_PRINTOUTS_LOADED                                       0x0001
#define D_XPM_API_PPSW_PRESENTING_PRINTOUTS                                   0x0002
#define D_XPM_API_PPSW_REJECTING_PRINTOUTS                                    0x0004
#define D_XPM_API_PPSW_PRINTOUTS_TAKEN                                        0x0008
#define D_XPM_API_PPSW_PRINTOUTS_REJECTED                                     0x0010
#define D_XPM_API_PPSW_PRESENTER_JAMMED                                       0x0020

//-- Printer ticket packet -> ticket status word 
#define D_XPM_API_PTSW_BUSY                                                   0x0001 // Ticket is still in process (printing, cutting, presenting, etc) 
#define D_XPM_API_PTSW_COMPLETED                                              0x0002 // Ticket has been printed, cutted and presented and either taken or rejected successfully  
#define D_XPM_API_PTSW_FAILED                                                 0x0004 // Ticket generation has failed (see other flags to determine where) 
#define D_XPM_API_PTSW_PRINTED                                                0x0008 // Ticket has been printed successfully  
#define D_XPM_API_PTSW_CUTTED                                                 0x0010 // Ticket has been cutted successfully  
#define D_XPM_API_PTSW_LOADED                                                 0x0020 // Ticket has been loaded into the presenter successfully 
#define D_XPM_API_PTSW_PRESENTED                                              0x0040 // Ticket(s) have been presented to the user successfully 
#define D_XPM_API_PTSW_EJECTED_OR_TAKEN                                       0x0080 // Ticket(s) have been ejected or taken by the user  
#define D_XPM_API_PTSW_REJECTED                                               0x0100 // Ticket(s) have been rejected into the paper bin 

//-- Configuration Data ->UART Baudrates  
#define D_XPM_API_CD_RS232_BAUDRATE_4800                                      0
#define D_XPM_API_CD_RS232_BAUDRATE_9600                                      1
#define D_XPM_API_CD_RS232_BAUDRATE_19200                                     2
#define D_XPM_API_CD_RS232_BAUDRATE_38400                                     3
#define D_XPM_API_CD_RS232_BAUDRATE_57600                                     4
#define D_XPM_API_CD_RS232_BAUDRATE_115200                                    5
#define D_XPM_API_CD_RS232_BAUDRATE_230400                                    6  /* Not supported */ 
#define D_XPM_API_CD_RS232_BAUDRATE_460800                                    7  /* Not supported */ 
#define D_XPM_API_CD_RS232_BAUDRATE_MAX                                       D_XPM_API_CD_RS232_BAUDRATE_115200 
#define D_XPM_API_CD_RS232_BAUDRATE_DEF                                       D_XPM_API_CD_RS232_BAUDRATE_115200

//-- Configuration Data ->UART parity on/off
#define D_XPM_API_CD_RS232_PARITY_OFF                                         0
#define D_XPM_API_CD_RS232_PARITY_ON                                          1

//-- Configuration Data ->UART parity odd/even (if on)
#define D_XPM_API_CD_RS232_PARITY_ODD                                         0
#define D_XPM_API_CD_RS232_PARITY_EVEN                                        1

//-- Configuration Data ->UART data bits (7/8)
#define D_XPM_API_CD_RS232_DATABIT_7                                          0  // Not supported 
#define D_XPM_API_CD_RS232_DATABIT_8                                          1

//-- Configuration Data ->UART stop bits (1/2)
#define D_XPM_API_CD_RS232_STOPBIT_1                                          0
#define D_XPM_API_CD_RS232_STOPBIT_2                                          1

//-- Configuration Data ->UART flow control (handshake)
#define D_XPM_API_CD_RS232_FLOWCNTL_NONE                                      0
#define D_XPM_API_CD_RS232_FLOWCNTL_HW                                        1
#define D_XPM_API_CD_RS232_FLOWCNTL_SW                                        2

//-- Configuration Data - Wide font setting
#define D_XPM_API_CD_WF_NONE                                                  0 /* Wide font disabled */
#define D_XPM_API_CD_WF_STANDARD_24X24                                        1 /* 24x24 Standard wide font 0xE000..0xFFFF */
#define D_XPM_API_CD_WF_GB3212_CHINESE_24X24                                  2 /* 24x24 GB3212 Chinese wide font 0xA000..0xAFFFF */
#define D_XPM_API_CD_WF_STANDARD_20X20                                        3 /* 20x20 Standard wide font 0xE000..0xFFFF  */
#define D_XPM_API_CD_WF_KS6501_KOREAN_20X20                                   4 /* 20x20 KS5601 Korean font 0xA000..0xAFFFF */

//-- Configuration Data ->Printer Options
#define D_XPM_API_CD_PRO_CUTTER_INIT_ON_POWER_ON                              0x00000001
#define D_XPM_API_CD_PRO_DISABLE_INQUIRE_TOP_OF_FORM_ON_INIT                  0x00000002
#define D_XPM_API_CD_PRO_DISABLE_UNJAM_PAPER_ON_INIT                          0x00000004
#define D_XPM_API_CD_PRO_INVALIDATE_PAPER_ON_INIT                             0x00000008
#define D_XPM_API_CD_PRO_INVALIDATE_PAPER_ON_DATA_LOSS                        0x00000010
#define D_XPM_API_CD_PRO_SETUP_SINGLE_PAPER_PATH_ONLY                         0x00000020
#define D_XPM_API_CD_PRO_ADD_CUSTOMER_FLAG_TO_USB_VERSION                     0x00000040
#define D_XPM_API_CD_PRO_PRINT_WHOLE_OBJECTS_ONLY                             0x00000080 

//-- Configuration Data ->Paper Path Options
#define D_XPM_API_CD_PPO_DISABLE_PRINT_INFO_ON_LOAD                           0x00000001
#define D_XPM_API_CD_PPO_DISABLE_CUT_ON_LOAD                                  0x00000002
#define D_XPM_API_CD_PPO_INVALIDATE_PAPER_ON_LOAD                             0x00000004
#define D_XPM_API_CD_PPO_DISABLE_RETRACT_ON_PAPER_END                         0x00000008
#define D_XPM_API_CD_PPO_INVALIDATE_PAPER_ON_PAPER_END                        0x00000010
#define D_XPM_API_CD_PPO_FORM_FEED_TO_MARK                                    0x00000020
#define D_XPM_API_CD_PPO_DISABLE_PAPER_LOAD_SENSOR_DETECT                     0x00000040
#define D_XPM_API_CD_PPO_DISABLE_PAPER_LOAD_BUTTON_CONTROL                    0x00000080
#define D_XPM_API_CD_PPO_REDUCE_PRINTOUT_TO_FORM_LENGTH                       0x00000100
#define D_XPM_API_CD_PPO_REDUCE_REVERSE_PAPER_MOVEMENT                        0x00000200
#define D_XPM_API_CD_PPO_WHOLE_INITIAL_FORM_ON_PAPER_LOAD                     0x00000400
#define D_XPM_API_CD_PPO_DISABLE_EJECT_ON_PAPER_END                           0x00000800
#define D_XPM_API_CD_PPO_PREPULL_FORM_BEFORE_PRINT                            0x00001000
#define D_XPM_API_CD_PPO_MARK_CUT_OFFSET_IS_RELATIVE_TO_MARK_CENTER           0x00002000

//-- Configuration Data ->Test page code 
#define D_XPM_API_CD_TPC_TEST_PAGE_1_MASK                                     0x0F
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DISABLED                                 0x00
#define D_XPM_API_CD_TPC_TEST_PAGE_1_BLANK_PAGE                               0x01
#define D_XPM_API_CD_TPC_TEST_PAGE_1_SENSOR_READINGS                          0x02
#define D_XPM_API_CD_TPC_TEST_PAGE_1_MIXED_PATTERN                            0x03
#define D_XPM_API_CD_TPC_TEST_PAGE_1_INVALIDATE                               0x04
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DIAGONAL_1                               0x05
#define D_XPM_API_CD_TPC_TEST_PAGE_1_DIAGONAL_2                               0x06
#define D_XPM_API_CD_TPC_TEST_PAGE_1_GRID_1                                   0x07
#define D_XPM_API_CD_TPC_TEST_PAGE_1_GRID_2                                   0x08
#define D_XPM_API_CD_TPC_TEST_PAGE_1_LINES_1                                  0x09
#define D_XPM_API_CD_TPC_TEST_PAGE_1_LINES_2                                  0x0A
#define D_XPM_API_CD_TPC_TEST_PAGE_1_RHOMBUS_1                                0x0B
#define D_XPM_API_CD_TPC_TEST_PAGE_1_RHOMBUS_2                                0x0C
#define D_XPM_API_CD_TPC_TEST_PAGE_2_MASK                                     0xF0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DISABLED                                 0x00
#define D_XPM_API_CD_TPC_TEST_PAGE_2_BLANK_PAGE                               0x10
#define D_XPM_API_CD_TPC_TEST_PAGE_2_SENSOR_READINGS                          0x20
#define D_XPM_API_CD_TPC_TEST_PAGE_2_MIXED_PATTERN                            0x30
#define D_XPM_API_CD_TPC_TEST_PAGE_2_INVALIDATE                               0x40
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DIAGONAL_1                               0x50
#define D_XPM_API_CD_TPC_TEST_PAGE_2_DIAGONAL_2                               0x60
#define D_XPM_API_CD_TPC_TEST_PAGE_2_GRID_1                                   0x70
#define D_XPM_API_CD_TPC_TEST_PAGE_2_GRID_2                                   0x80
#define D_XPM_API_CD_TPC_TEST_PAGE_2_LINES_1                                  0x90
#define D_XPM_API_CD_TPC_TEST_PAGE_2_LINES_2                                  0xA0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_RHOMBUS_1                                0xB0
#define D_XPM_API_CD_TPC_TEST_PAGE_2_RHOMBUS_2                                0xC0

//-- Configuration Data ->External LED control 
#define D_XPM_API_CD_EXTLED_DISABLED                                          0
#define D_XPM_API_CD_EXTLED_ONBOARD_LEDS                                      1
#define D_XPM_API_CD_EXTLED_STATE_INDICATOR                                   2

//-- Configuration data - customer sensors - paper pre end sensor A #1 - enumeration
#define D_XPM_API_CD_CS_PPEA1_MASK                                            0x00000003
#define D_XPM_API_CD_CS_PPEA1_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA1_REFLEX                                          0x00000001
#define D_XPM_API_CD_CS_PPEA1_TLIGHT                                          0x00000002
#define D_XPM_API_CD_CS_PPEA1_SWITCH                                          0x00000003
#define D_XPM_API_CD_CS_PPEA1_ACTIVE_WHEN_LOW                                 0x00000008
#define D_XPM_API_CD_CS_PPEA1_PASS_THROUGH                                    0x00000004

//-- Configuration data - customer sensors - paper pre end sensor A #2 - enumeration
#define D_XPM_API_CD_CS_PPEA2_MASK                                            0x00000030
#define D_XPM_API_CD_CS_PPEA2_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA2_REFLEX                                          0x00000010
#define D_XPM_API_CD_CS_PPEA2_TLIGHT                                          0x00000020
#define D_XPM_API_CD_CS_PPEA2_SWITCH                                          0x00000030
#define D_XPM_API_CD_CS_PPEA2_ACTIVE_WHEN_LOW                                 0x00000080
#define D_XPM_API_CD_CS_PPEA2_PASS_THROUGH                                    0x00000040

//-- Configuration data - customer sensors - paper pre end sensor A #3 - enumeration
#define D_XPM_API_CD_CS_PPEA3_MASK                                            0x00000300
#define D_XPM_API_CD_CS_PPEA3_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEA3_REFLEX                                          0x00000100
#define D_XPM_API_CD_CS_PPEA3_TLIGHT                                          0x00000200
#define D_XPM_API_CD_CS_PPEA3_SWITCH                                          0x00000300
#define D_XPM_API_CD_CS_PPEA3_ACTIVE_WHEN_LOW                                 0x00000800
#define D_XPM_API_CD_CS_PPEA3_PASS_THROUGH                                    0x00000400

//-- Configuration data - customer sensors - paper pre end sensor B #1 - enumeration
#define D_XPM_API_CD_CS_PPEB1_MASK                                            0x00003000
#define D_XPM_API_CD_CS_PPEB1_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB1_SWITCH                                          0x00001000
#define D_XPM_API_CD_CS_PPEB1_ACTIVE_WHEN_LOW                                 0x00008000
#define D_XPM_API_CD_CS_PPEB1_PASS_THROUGH                                    0x00004000

//-- Configuration data - customer sensors - paper pre end sensor B #2 - enumeration
#define D_XPM_API_CD_CS_PPEB2_MASK                                            0x00030000
#define D_XPM_API_CD_CS_PPEB2_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB2_SWITCH                                          0x00010000
#define D_XPM_API_CD_CS_PPEB2_ACTIVE_WHEN_LOW                                 0x00080000
#define D_XPM_API_CD_CS_PPEB2_PASS_THROUGH                                    0x00040000

//-- Configuration data - customer sensors - paper pre end sensor B #3 - enumeration
#define D_XPM_API_CD_CS_PPEB3_MASK                                            0x00300000
#define D_XPM_API_CD_CS_PPEB3_NONE                                            0x00000000
#define D_XPM_API_CD_CS_PPEB3_SWITCH                                          0x00100000
#define D_XPM_API_CD_CS_PPEB3_ACTIVE_WHEN_LOW                                 0x00800000
#define D_XPM_API_CD_CS_PPEB3_PASS_THROUGH                                    0x00400000

//-- Configuration data - customer sensors - auxiliary sensor - enumeration
#define D_XPM_API_CD_CS_AUX_MASK                                              0x07000000
#define D_XPM_API_CD_CS_AUX_NONE                                              0x00000000
#define D_XPM_API_CD_CS_AUX_REFLEX                                            0x01000000
#define D_XPM_API_CD_CS_AUX_TLIGHT                                            0x02000000
#define D_XPM_API_CD_CS_AUX_SWITCH                                            0x03000000
#define D_XPM_API_CD_CS_AUX_ACTIVE_WHEN_LOW                                   0x08000000

//-- Configuration data - customer sensors - reserved
#define D_XPM_API_CD_CS_RES_MASK                                              0xFFFF0000

//-- Number of Paper Paths
#define D_XPM_API_NR_OF_PAPER_PATHS                                           3

/*
** TYPES
*/

//-- Printer packet header
#pragma pack(1) 
typedef struct 
   { unsigned char                   sync_1;                                 // First synchronization byte (must be 0xAA)
     unsigned char                   sync_2;                                 // Second synchronization byte (must be 0x55)
     unsigned char                   id;                                     // Identification of packet 
     unsigned char                   length;                                 // Length of data part
   } TS_XPM_PCKT_HEADER;                                                     // Packet header
#pragma pack() 

//-- Printer status packet 
#pragma pack(1) 
typedef struct  
   { uint32_t                        status;                                 // Printer status dword bits as in D_XPM_PSDW_...
     unsigned char                   parameter;                              // Print parameter
     short                           print_head_temp_c;                      // Printhead temperature in degrees celcius
     unsigned short                  board_voltage_tv;                       // Board voltage in tenths of volt 
     unsigned char                   state;                                  // Status of paper control
     unsigned short                  error_code;                             // Error code of paper control (WHAT/WHEN) as in D_XPM_PECW_...
     unsigned short                  error_info;                             // Error info of paper control (ENUM)
   } TS_XPM_API_PCKT_PRINTER_STATUS;                                         // Printer status packet
#pragma pack() 

//-- Printer info packet
#pragma pack(1) 
typedef struct 
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     char                            part_nr[16+1];                          // Part number, inluding 0 terminator
     char                            serial_nr[16+1];                        // Serial number, inluding 0 terminator
     char                            date_of_manuf[16+1];                    // Date of manufacturing, inluding 0 terminator
     uint32_t                        model_code;                             // Model   code as in D_XPM_MC_.... (see exo_system.h)
     uint32_t                        config_code;                            // Config  code as in D_XPM_CC_.... (see exo_system.h)
     uint32_t                        sensor_code;                            // Sensor  code as in D_XPM_SC_.... (see exo_system.h)
     uint32_t                        feature_code;                           // Feature code as in D_XPM_FC_.... (see exo_system.h)
     char                            firmware_part_nr[16+1];                 // Firmware part number, inluding 0 terminator
     char                            firmware_version[10+1];                 // Firmware version  Rx-Vy.zzb
     char                            firmware_date[12+1];                    // Firmware date     Mmm dd yyyy (ANSI standard).
     uint32_t                        firmware_patch;                         // Firmware pach as in D_XPM_MC_FWPATCH_... (see exo_settings.h
     unsigned char                   reserved[4];                            // Reserved bytes (2 uint32_ts, zero initialized)
   } TS_XPM_API_PCKT_PRINTER_INFO;                                           // Printer info packet
#pragma pack() 

//-- Printer sensor packet
#pragma pack(1) 
typedef struct
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     unsigned char                   adc_measurement[16];                    // ADC channel #00..#15 measurements in domain 0=0V..255=5V                          (ADC#XX)
     unsigned short                  digital_input;                          // DIN digital inputs bits #00..#15                                                  (DIN#XX)
     short                           print_head_temp_c;                      // Printhead temperature in degrees celcius                                          (ADC#02)
     unsigned short                  board_voltage_tv;                       // Board voltage in tenths of volt                                                   (ADC#01)
     unsigned char                   paper_pre_end_sensor_A_state[3];        // State byte for paper pre end sensor A     (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#03/06/--)
     unsigned char                   paper_pre_end_sensor_B_state[3];        // State byte for paper pre end sensor B     (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (DIN#06/07/--)
     unsigned char                   paper_path_key_state[3];                // State byte for paper path keys            (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (DIN#04/05/--)
     unsigned char                   paper_path_entry_sensor_state[3];       // State byte for paper path entry sensors   (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#08/09/--)
     unsigned char                   paper_path_exit_sensor_state[3];        // State byte for paper path exit sensors    (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#10/11/--)
     unsigned char                   paper_entry_sensor_1_state;             // State byte for paper entry sensor #1      (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#04)
     unsigned char                   paper_entry_sensor_2_state;             // State byte for paper entry sensor #2      (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#05)
     unsigned char                   head_up_sensor_state;                   // State byte for head up sensor             (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (DIN#00)
     unsigned char                   cutter_sensor_1_state;                  // State byte for cutter sensor #1 (left)    (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (DIN#01)
     unsigned char                   cutter_sensor_2_state;                  // State byte for cutter sensor #2 (right)   (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (DIN#02)
     unsigned char                   paper_exit_sensor_state;                // State byte for paper exit sensor          (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#00)
     unsigned char                   auxiliary_sensor_state;                 // State byte for auxiliary sensor           (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#12)
     unsigned char                   presenter_sensor_1_state;               // State byte for presenter sensor #1        (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#13)
     unsigned char                   presenter_sensor_2_state;               // State byte for presenter sensor #2        (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#14)
     unsigned char                   presenter_sensor_3_state;               // State byte for presenter sensor #3        (D_XPM_PCKT_TX_PRINTER_SENSOR_STATE_..) (ADC#15)
     unsigned char                   reserved[5];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_SENSOR;                                         // Printer sensor packet
#pragma pack() 

//-- Printer flash packet
#pragma pack(1) 
typedef struct 
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     char                            info[6][20+1];                          // Flash area info, indexed by D_XPM_PCKT_TX_PRINTER_FLASH_AREA_INDEX_..., inluding 0 terminator
     unsigned short                  checksum[6][2];                         // Flash area 16 bit checksum, indexed by D_XPM_PCKT_TX_PRINTER_FLASH_AREA_INDEX_..., 0=stored and 1=calculated
     unsigned char                   state[6];                               // Flash area state, indexed by D_XPM_PCKT_TX_PRINTER_FLASH_AREA_INDEX_..., 
     unsigned char                   reserved[6];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_FLASH;                                          // Printer flash packet
#pragma pack() 

//-- TS_XPM_API_PCKT_PRINTER_FEED - Printer feed packet - byte aligned
#pragma pack(1) 
typedef struct 
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     unsigned char                   state;                                  // State of the feed unit control 
     unsigned char                   nr_of_paper_paths;                      // Total number of paper path
     unsigned short                  paper_path_status[3];                   // Status word of each paper path as in D_XPM_PCKT_TX_PRINTER_FEED_PATH_PFSW_...
     unsigned char                   paper_path_state[3];                    // State of the paper path control for each paper path 
     unsigned char                   reserved[8];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_FEED;                                           // Printer feed packet
#pragma pack() 

//-- TS_XPM_API_PCKT_PRINTER_PRESENT - Presenter status packet - byte aligned
#pragma pack(1) 
typedef struct  
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     unsigned char                   type;                                   // Type of presenter (0 is NONE)
     unsigned short                  status;                                 // Presenter status word bits as in D_XPM_PCKT_TX_PRINTER_PRESENTER_PSW_...
     unsigned char                   state;                                  // State of presenter control (idle/loading/presenting/rejecting, etc.)
     unsigned char                   nr_of_loaded_printouts;                 // Total number of currently loaded  (bundled) printouts
     uint32_t                        total_nr_of_loaded_printouts;           // Total number of loaded    printouts since start / power on
     uint32_t                        total_nr_of_taken_printouts;            // Total number of taken     printouts since start / power on
     uint32_t                        total_nr_of_rejected_printouts;         // Total number of rejected  printouts since start / power on
     unsigned char                   reserved[3];                            // Reserved bytes, must be 0
   } TS_XPM_API_PCKT_PRINTER_PRESENT;                                        // Printer present packet
#pragma pack() 

//-- Printer life cycle data packet
#pragma pack(1) 
typedef struct 
   { unsigned char                   protocol_revision;                      // Revision of communication protocol
     uint32_t                        time_powered_s;                         // Total time that the printer is powered in sec
     uint32_t                        distance_stepped_mm;                    // Total distance stepped in mm
     uint32_t                        distance_burned_mm;                     // Total distance burned in mm
     uint32_t                        paper_transported_mm;                   // Total length of transported paper in mm
     uint32_t                        nr_of_system_starts;                    // Total number of system starts
     uint32_t                        nr_of_hard_resets;                      // Total number of hardware resets 
     uint32_t                        nr_of_paper_cuts;                       // Total number of paper cuts 
     unsigned short                  nr_of_paper_loads;                      // Total number of paper loads 
     unsigned char                   reserved[8];                            // Reserved bytes (2 DWORDs, zero initialized)
   } TS_XPM_API_PCKT_PRINTER_LIFE;                                           // Printer life cycle data packet
#pragma pack() 

//-- XPM configuration data 
#pragma pack(1)
typedef struct 
{    unsigned short                  revision_number;                                                  // Structure revision number 
     unsigned short                  size_of_block;                                                    // Size of this structure
     char                            cust_serial_nr[16+1];                                             // Customer-serial number, zero is unused
     unsigned char                   reserved_1[17];                                                   // Reserved bytes for future use (must be 0)
     unsigned char                   cust_flags;                                                       // Customer specific flags
     unsigned char                   baudrate;                                                         // Baudrate matching one of D_XPM_API_CD_RS232_BAUDRATE_...
     unsigned char                   parity_on_off;                                                    // Parity matching one of D_XPM_API_CD_RS232_PARITY_... (on/off)
     unsigned char                   parity;                                                           // Parity matching one of D_XPM_API_CD_RS232_PARITY_... (even/odd, if on)
     unsigned char                   data_bits;                                                        // Data bits matching one of D_XPM_API_CD_RS232_DATABIT_... (7/8)
     unsigned char                   stop_bits;                                                        // Stop bits matching one of D_XPM_API_CD_RS232_STOPBIT_... (1/2)
     unsigned char                   flow_control;                                                     // Flow control matching one of D_XPM_API_CD_RS232_FLOWCNTL_... (SW/HW/NO)
     unsigned char                   block_host_tx_on_paper_out;                                       // Flag indicating disable host TX on paper out (RS232-HW/SW flow control only)
     unsigned char                   wide_font;                                                        // Enumeration indicating wide font set enabled if non zero
     uint32_t                        printer_options;                                                  // Customer Printer options
     uint32_t                        customer_sensors;                                                 // Customer auxiliary and paper pre end sensors as in XPM_API_CD_CS_....
     unsigned short                  print_speed_tmms[D_XPM_API_NR_OF_PAPER_PATHS];                    // Print speed (default) in 1/10 mm/sec
     unsigned short                  paper_load_speed_tmms[D_XPM_API_NR_OF_PAPER_PATHS];               // Paper load speed in 1/10 mm/s
     unsigned short                  paper_control_speed_tmms[D_XPM_API_NR_OF_PAPER_PATHS];            // Paper control speed in 1/10 mm/s
     char                            print_density[D_XPM_API_NR_OF_PAPER_PATHS];                       // Print density (default) in percent
     unsigned char                   dhis_burn_fac[D_XPM_API_NR_OF_PAPER_PATHS];                       // Dot history burn factor in percent of total burn time (zero indicates OFF)
     unsigned char                   multi_strobe_fac[D_XPM_API_NR_OF_PAPER_PATHS];                    // Multi strobe counter, zero is undefined
     char                            burn_time_correction_us[D_XPM_API_NR_OF_PAPER_PATHS][11];         // Burn time corrections in us for -30�..+70�C with 10�C interval
     uint32_t                        paper_path_options[D_XPM_API_NR_OF_PAPER_PATHS];                  // Customer Paper Path options
     uint32_t                        form_length_ym[D_XPM_API_NR_OF_PAPER_PATHS];                      // Maximum form length in micrometer
     unsigned short                  mark_length_ym[D_XPM_API_NR_OF_PAPER_PATHS];                      // Length of a black/hole mark in micrometer
     uint32_t                        mark_cut_dis_ym[D_XPM_API_NR_OF_PAPER_PATHS];                     // Distance from end of black/hole mark to cut position in micrometer
     unsigned short                  paper_end_detect_dis_ym[D_XPM_API_NR_OF_PAPER_PATHS];             // Minimum distance required before detecting paper end
     unsigned char                   paper_park_pos[D_XPM_API_NR_OF_PAPER_PATHS];                      // Paper park position 0 or 1
     short                           present_offset_ym[D_XPM_API_NR_OF_PAPER_PATHS];                   // Presentation offset in micrometer
     uint32_t                        paper_load_feed_ym[D_XPM_API_NR_OF_PAPER_PATHS];                  // Feed length after paper load in micrometer
     unsigned char                   paper_load_duration_s[D_XPM_API_NR_OF_PAPER_PATHS];               // Maximum duration of paper load in seconds
     unsigned short                  paper_weight_gm2[D_XPM_API_NR_OF_PAPER_PATHS];                    // Paper weight in gramm per square meter
     uint32_t                        paper_width_ym[D_XPM_API_NR_OF_PAPER_PATHS];                      // paper width in 1/10 mm
     unsigned char                   print_data_lost_timeout_s;                                        // Timeout in sec for print data to be regarded lost 
     unsigned char                   external_led_control;                                             // External LED control as in D_XPM_API_CD_EXTLED_... 
     unsigned short                  present_speed_tmms[D_XPM_API_NR_OF_PAPER_PATHS];                  // Present speed in 1/10 mm/s (only for particular presenters)
     unsigned short                  bundler_speed_tmms[D_XPM_API_NR_OF_PAPER_PATHS];                  // Bundler speed in 1/10 mm/s (only for particular presenters)
     short                           mark_cut_correction_ym[D_XPM_API_NR_OF_PAPER_PATHS];              // Mark cut correction in micrometer
     short                           paper_bundle_correction_ym[D_XPM_API_NR_OF_PAPER_PATHS];          // Paper bundle correction in micrometer (only for particular presenters)
     unsigned char                   test_page_code[D_XPM_API_NR_OF_PAPER_PATHS];                      // Code for printout of test page #1 and #2 on paper load
     short                            print_correction_ym[D_XPM_API_NR_OF_PAPER_PATHS];                 // Correction of top of paper at start if print (unparking) in micrometer
     unsigned char                   nr_of_cut_retries;                                                // Total number of retries after a cut failure has been detected
     unsigned char                   test_page_code_on_close[D_XPM_API_NR_OF_PAPER_PATHS];             // Code for printout of test page #1 and #2 on printer close
     unsigned char                   dhis_level[D_XPM_API_NR_OF_PAPER_PATHS];                          // Total numbr of dot history level (0 = dot history disabled)
     unsigned char                   initialization_delay_ts;                                          // Initialization delay in 1/10 seconds
     char 			     acMarkThresholdCorrTv[3];					   	// Mark Threashold correction		
     unsigned char                   reserved2[2];                                                     // Reserved bytes for future use (complete structure is 0xFF bytes) (must be 0)
     unsigned short                  checksum;                                                         // 16 Bit checksum of structure
} TS_XPM_API_PCKT_CONFIG_DATA;                                                                         // XPM configuration data structure as uploaded to and stored in flash
#define D_XPM_API_PCKT_CONFIG_DATA_REV 1                                                               // Revision of configuration data
#pragma pack()

//-- XPM data exchange - upload block types 
typedef enum
   { EV_XPM_API_ET_FIRMWARE                      = 0x00 
   , EV_XPM_API_ET_FONT_1                        = 0x01
   , EV_XPM_API_ET_LOGO                          = 0x02
   , EV_XPM_API_ET_FONT_2                        = 0x03
   , EV_XPM_API_ET_DATA                          = 0x04
   , EV_XPM_API_ET_WIDE_FONT                     = 0x05
   , EV_XPM_API_ET_IMAGES                        = 0x06  
   , EN_XPM_API_EXCHANGE_TYPE                    = 7
   } ET_XPM_API_EXCHANGE_TYPE;                                               // Identification of all exchange type (contents)

typedef enum
   { EV_XPM_API_ER_OK                            = 0x00
   , EV_XPM_API_ER_ERROR_PROTOCOL_NOT_SUPPORTED  = 0x01
   , EV_XPM_API_ER_ERROR_MODE_NOT_SUPPORTED      = 0x02
   , EV_XPM_API_ER_ERROR_PACKET_SIZE_MISMATCH    = 0x03
   , EV_XPM_API_ER_ERROR_DATA_TYPE_MISMATCH      = 0x04
   , EV_XPM_API_ER_ERROR_DATA_SIZE_MISMATCH      = 0x05
   , EV_XPM_API_ER_ERROR_DATA_CRC_MISMATCH       = 0x05
   , EV_XPM_API_ER_ERROR_DATA_MISSING            = 0x08
   , EV_XPM_API_ER_ERROR_EXCHANGE_ID_MISMATCH    = 0x06
   , EV_XPM_API_ER_ERROR_NO_EXCHANGE_ACTIVE      = 0x07
   , EV_XPM_API_ER_ERROR_FLASH_FAILURE           = 0x08
   } ET_XPM_TB_EXCHANGE_RESULT;                                                               // Enumeration of all exchange result values

//-- XPM data exchange - exchange file header
#pragma pack(1)
typedef struct 
   { unsigned short                  size_hdr;                               // Size of the file header in bytes
     unsigned short                  revision_nr;                            // Revision of file : 0x00=Initial revision
     unsigned short                  type_mask;                              // Contents of file : see ET_XPM_API_EXCHANGE_TYPE, used as bitmask
     uint32_t                        size;                                   // Size of all blocks in bytes (without file header / footer)
     uint32_t                        date;                                   // Data at which the file was generated
     uint32_t                        time;                                   // Time at which the file was generated
     char                            info[256+1];                            // Information regarding file contents
     unsigned char                   reserved[32];                           // 32 reserved bytes, must be 0
   } TS_XPM_API_EXCHANGE_FILE_HEADER;                                        // XPM exchange file header
#define D_XPM_API_EXCHANGE_FILE_REVISION   1
#pragma pack()

//-- XPM data exchange - exchange file footer
#pragma pack(1)
typedef struct 
   { uint32_t                   checksum;                                    // 32 Bit checksum of complete file inclusive header exclusive footer
   } TS_XPM_API_EXCHANGE_FILE_FOOTER;                                        // XPM exchange file footer
#pragma pack()

//-- XPM data exchange - exchange block header 
#pragma pack(1)
typedef struct 
   { unsigned short                  type;                                   // Contents of block : see ET_XPM_API_EXCHANGE_TYPE;
     uint32_t                        size;                                   // Size of the block in bytes (without block header / footer)
     char                            info[20+1];                             // Information regarding block contents
     unsigned char                   reserved[5];                            // 5 reserved bytes, must be 0
   } TS_XPM_API_EXCHANGE_BLOCK_HEADER;                                       // XPM exchange block footer
#pragma pack()

//-- XPM data exchange - exchange block footer
#pragma pack(1)
typedef struct 
   { unsigned short                  checksum;                               // 16 bit checksum of the blocks inclusive header exclusive footer
   } TS_XPM_API_EXCHANGE_BLOCK_FOOTER;                                       // XPM exchange block footer
#pragma pack()

//-- XPM data exchange - upload start request packet
#pragma pack(1)
typedef struct 
   { unsigned char                   protocol;                               // Data exchange protocol (0 indicates default)
     unsigned char                   mode;                                   // Data transfer mode     (0 indicates not compressed)
     unsigned short                  type_mask;                              // Type of data to upload (bitmask from ET_XPM_API_DATA_EXCHANGE_TYPE)
     uint32_t	                     size;                                   // Size of data to upload
   } TS_XPM_API_PCKT_EXCHANGE_UPLOAD_START_REQUEST;                          // Data exchange upload start request packet
#pragma pack()

//-- XPM data exchange - upload start response packet
#pragma pack(1)
typedef struct 
   { unsigned char                   result;                                 // Result of data upload request
   } TS_XPM_API_PCKT_EXCHANGE_UPLOAD_START_RESPONSE;                         // Data exchange upload start response packet
#pragma pack()

//-- XPM data exchange - upload end request packet
#pragma pack(1)
typedef struct 
   { unsigned short                  checksum;                               // 16 bit Checksum of data to upload
   } TS_XPM_API_PCKT_EXCHANGE_UPLOAD_END_REQUEST;                            // Data exchange upload end request packet
#pragma pack()

//-- XPM data exchange - upload end response packet
#pragma pack(1)
typedef struct 
   { unsigned char                   result;                                 // Result of data upload request
   } TS_XPM_API_PCKT_EXCHANGE_UPLOAD_END_RESPONSE;                           // Data exchange upload end response packet
#pragma pack()

/*
** VARIABLES
*/

/*
** FUNCTIONS
*/

//-- Inquiry function 
char *         xpm_api_get_library_info    (void);

//-- Printer communication interface
intptr_t       xpm_api_printer_open        (char * port); // e.g. port = "/dev/usb/lp0" or port = "/dev/bus/usb/003/002"
int            xpm_api_printer_set_serial  (intptr_t printer, unsigned long baud_rate, unsigned char data_bits, unsigned char parity, unsigned char stop_bits);
void           xpm_api_printer_close       (intptr_t printer);
int            xpm_api_printer_write       (intptr_t printer, unsigned char * data, int size, unsigned long timeout_ms);
int            xpm_api_printer_read        (intptr_t printer, unsigned char * data, int size, unsigned long timeout_ms);
int            xpm_api_printer_clear       (intptr_t printer);

#ifdef __cplusplus
}
#endif

#endif // __XPM_API_H
